﻿using System.Linq;
using System.Windows;
using System.Windows.Controls;
using Telerik.Windows;
using Telerik.Windows.Controls;

namespace TreeViewInComboBox
{
	public class DropDownButtonTree:RadDropDownButton
	{
		private RadTreeView treeView;

		static DropDownButtonTree()
		{
			EventManager.RegisterClassHandler(typeof(DropDownButtonTree),
				RadDropDownButton.DropDownOpeningEvent,
				new RoutedEventHandler(OnDropDownOpening));
		}

		private void OnTreeViewSelectionChanged(object sender, Telerik.Windows.Controls.SelectionChangedEventArgs e)
		{
			this.UpdateButtonFromTreeViewSelection();
		}

		private void UpdateButtonFromTreeViewSelection()
		{
			if (treeView != null)
			{
				if (treeView.SelectedItem != null)
				{
					this.Content = treeView.SelectedItem;

					// Try find an appropriate template:
					if (treeView.SelectedContainer != null)
					{
						this.ContentTemplate = treeView.SelectedContainer.HeaderTemplate;
					}
					else
					{
						//Try guess the template. This will work in 80% of the scenarions.
						//It will not work when the template of an item depends on the hierarchy.
						if (treeView.ItemTemplateSelector != null && treeView.ItemTemplate == null)
						{
							this.ContentTemplate = treeView.ItemTemplateSelector.SelectTemplate(this.Content, this);
						}
						else
						{
							this.ContentTemplate = treeView.ItemTemplate;
						}
					}
				}
				else
				{
					this.Content = null;
				}
			}
		}

		static void OnDropDownOpening(object sender, RoutedEventArgs e)
		{
			var button = sender as DropDownButtonTree;

			(button.DropDownContent as FrameworkElement).Visibility = Visibility.Visible;

			// Adjust the content width:
			var content = button.DropDownContent as FrameworkElement;
			content.MinWidth = button.ActualWidth - 2;

			// Set selection if needed:
			if (button.treeView != null)
			{
				button.treeView.SelectedItem = button.Content;
			}

			// Trasfer the Button DataContext to the DropDown
			var visualDropDown = button.DropDownContent as FrameworkElement;
			visualDropDown.DataContext = button.DataContext;
		}

		protected override void OnContentChanged(object oldContent, object newContent)
		{
			base.OnContentChanged(oldContent, newContent);

			if (treeView != null)
			{
				treeView.SelectedItem = newContent;
			}
		}

		protected override void OnDropDownContentChanged(object oldValue, object newValue)
		{
			base.OnDropDownContentChanged(oldValue, newValue);

			if (treeView != null)
			{
				treeView.SelectionChanged -= OnTreeViewSelectionChanged;
			}

			treeView = null;
			var visualContent = newValue as FrameworkElement;
			if (visualContent != null)
			{
				treeView = visualContent as RadTreeView ?? visualContent.ChildrenOfType<RadTreeView>().FirstOrDefault();
			}
			if (treeView != null)
			{
				treeView.SelectionChanged += OnTreeViewSelectionChanged;
			}
		}
	}
}
